"""
config
===============================================================================
"""
import wx.stc as stc
from wbBase.control.textEditControl import PyTextEditConfig
from wbBase.dialog.preferences import PreferencesPageBase


class NameSpaceConfig(PyTextEditConfig):
    def __init__(self):
        PyTextEditConfig.__init__(self)
        self.ShowLineNumbers = False
        self.WrapMode = stc.STC_WRAP_WORD

    def appendProperties(self, page):
        """
        Append properties to PreferencesPage
        """
        self.registerPropertyEditors(page)
        self.appendProperties_main(page)
        self.appendProperties_caret(page)
        self.appendProperties_selection(page)
        # self.appendProperties_indentation(page)
        # self.appendProperties_line_ending(page)
        self.appendProperties_line_warp(page)
        # self.appendProperties_line_numbers(page)
        # self.appendProperties_code_folding(page)
        self.appendProperties_syntax_colour(page)


class NameSpacePreferences(PreferencesPageBase):
    """
    Additional page for the preferences dialog of the
    Workbench application.
    """
    def __init__(self, parent):
        PreferencesPageBase.__init__(self, parent)
        from . import name, namespaceConfig

        self.name = name
        self._config = namespaceConfig
        self.config.appendProperties(self)

    @property
    def config(self):
        return self._config

    def applyValues(self):
        """
        Apply configuration to the namespace panel.
        """
        pane = self.app.TopWindow.panelManager.getPaneByCaption(self.name)
        if pane:
            self.config.getPropertyValues(self)
            win = pane.window
            tree = win.tree
            tree.SetBackgroundColour(self.config.backgroundColour.ChangeLightness(130))
            tree.SetForegroundColour(self.config.foregroundColour)
            self.config.apply(win.text)

    def saveValues(self):
        self.applyValues()
        self.config.save()



"""
control
===============================================================================
"""
from __future__ import annotations

from typing import TYPE_CHECKING

import wx
from wbBase.control.filling import Filling

if TYPE_CHECKING:
    from .config import NameSpaceConfig


class NameSpace(Filling):
    def __init__(
        self,
        parent: wx.Window,
        id: int = wx.ID_ANY,
        pos: wx.Point = wx.DefaultPosition,
        size: wx.Size = wx.DefaultSize,
        style: int = wx.SP_LIVE_UPDATE
        | wx.SP_NOBORDER
        | wx.SP_NO_XP_THEME
        | wx.NO_BORDER,
    ):
        super().__init__(
            parent,
            id,
            pos,
            size,
            style,
            name="NameSpace",
            rootObject=None,
            rootLabel="NameSpace",
            rootIsNamespace=True,
            static=False,
        )
        self.config.load()

    @property
    def config(self) -> NameSpaceConfig:
        """
        Configuration of the namespace panel.
        """
        from . import namespaceConfig

        return namespaceConfig



"""
wbpNamespace
===============================================================================

Tree view to inspect the namespace of the application
"""

from __future__ import annotations

from typing import TYPE_CHECKING

import wx
from wx import aui

from .config import NameSpaceConfig, NameSpacePreferences
from .control import NameSpace

if TYPE_CHECKING:
    from wbBase.application import App

__version__ = "0.1.4"

name = "Namespace"

namespaceConfig = NameSpaceConfig()

nameSpaceInfo = aui.AuiPaneInfo()
nameSpaceInfo.Name(name)
nameSpaceInfo.Caption(name)
nameSpaceInfo.MaximizeButton(True)
nameSpaceInfo.MinimizeButton(True)
nameSpaceInfo.CloseButton(False)
nameSpaceInfo.Right()
nameSpaceInfo.Dock()
nameSpaceInfo.Hide()
nameSpaceInfo.Resizable()
nameSpaceInfo.MinSize(150, 100)
nameSpaceInfo.BestSize(250, 200)
nameSpaceInfo.Icon(wx.ArtProvider.GetBitmap("NAMESPACE", wx.ART_FRAME_ICON))


panels = [(NameSpace, nameSpaceInfo)]
preferencepages = [NameSpacePreferences]


def loadConfig(app: App):
    namespaceConfig.load()
